import StringIO
import unittest
import os
import tempfile
import time
import shutil
from hepdata_converter import convert
from hepdata_converter.testsuite.test_writer import WriterTestSuite


class CSVWriterTestCase(WriterTestSuite):
    def setUp(self):
        super(CSVWriterTestCase, self).setUp()
        self.table_csv = (
            '#: name: Table 1\n'
            '#: description: The measured fiducial cross sections. The first systematic uncertainty is the combined systematic uncertainty excluding luminosity, the second is the luminosity\n'
            '#: data_file: data1.yaml\n'
            '#: keyword reactions: P P --> Z0 Z0 X\n'
            '#: keyword observables: SIG\n'
            '#: keyword energies: 7000\n'
            '#: RE\tP P --> Z0 < LEPTON+ LEPTON- > Z0 < LEPTON+ LEPTON- > X\n'
            'SQRT(S) IN GEV\tSIG(fiducial) IN FB\tstat +\tstat -\tsys +\tsys -\tsys,lumi +\tsys,lumi -\n'
            '7000\t25.4\t3.3\t-3.0\t1\t-1.2\t1\t-1\n'
            '8000\t29.8\t3.8\t-3.5\t1.7\t-1.5\t1.2\t1.2\n'
            '9000\t12.7\t3.1\t-2.9\t1.7\t1.7\t0.5\t0.5\n'
        )
        self.table_2_csv = (
            '#: name: Table 9\n'
            '#: description: The observed and expected EmissT distribution in the dielectron SR-Z. The negigible estimated contribution from Z+jets is omitted in these distributions. The last bin contains the overflow.\n'
            '#: data_file: data9.yaml\n'
            '#: keyword energies: 8000\n'
            '#: SQRT(S)\t\t8000.0\t8000.0\t\t\t8000.0\t8000.0\n'
            '#: EVENTS\t\t25\t25\t\t\t25\t25\n'
            'ETMISS IN GEV LOW\tETMISS IN GEV HIGH\tData\tExpected Background\tstat +\tstat -\tGGM 700 200 1.5\tGGM 900 600 1.5\n'
            '200.0\t225.0\t0.0\t0.0\t0.0\t0.0\t0.0\t0.0\n'
            '225.0\t250.0\t6.0\t0.95\t0.41\t-0.51\t6.46\t0.97\n'
            '250.0\t275.0\t1.0\t0.9\t0.41\t-0.26\t6.82\t1.07\n'
            '275.0\t300.0\t1.0\t0.42\t0.12\t-0.19\t2.82\t1.17\n'
            '300.0\t325.0\t1.0\t0.34\t0.16\t-0.15\t2.41\t1.05\n'
            '325.0\t350.0\t2.0\t0.07\t0.19\t-0.16\t3.11\t1.08\n'
            '350.0\t375.0\t1.0\t0.68\t0.56\t-0.55\t0.7\t1.13\n'
            '375.0\t400.0\t1.0\t0.17\t0.1\t-0.15\t0.9\t1.2\n'
            '400.0\t425.0\t0.0\t0.24\t0.11\t-0.1\t0.69\t1.01\n'
            '425.0\t450.0\t1.0\t0.01\t0.08\t0.08\t0.72\t0.94\n'
            '450.0\t475.0\t0.0\t0.3\t0.33\t0.33\t0.0\t0.88\n'
            '475.0\t500.0\t2.0\t0.16\t0.17\t-0.14\t0.93\t4.59\n'
        )

        self.table_2_packed_csv = (
            '#: name: Table 9\n'
            '#: description: The observed and expected EmissT distribution in the dielectron SR-Z. The negigible estimated contribution from Z+jets is omitted in these distributions. The last bin contains the overflow.\n'
            '#: data_file: data9.yaml\n'
            '#: keyword energies: 8000\n'
            '#: SQRT(S)\t\t8000.0\n'
            '#: EVENTS\t\t25\n'
            'ETMISS IN GEV LOW\tETMISS IN GEV HIGH\tData\n'
            '200.0\t225.0\t0.0\n'
            '225.0\t250.0\t6.0\n'
            '250.0\t275.0\t1.0\n'
            '275.0\t300.0\t1.0\n'
            '300.0\t325.0\t1.0\n'
            '325.0\t350.0\t2.0\n'
            '350.0\t375.0\t1.0\n'
            '375.0\t400.0\t1.0\n'
            '400.0\t425.0\t0.0\n'
            '425.0\t450.0\t1.0\n'
            '450.0\t475.0\t0.0\n'
            '475.0\t500.0\t2.0\n'
            '\n'
            '#: SQRT(S)\t\t8000.0\n'
            '#: EVENTS\t\t25\n'
            'ETMISS IN GEV LOW\tETMISS IN GEV HIGH\tExpected Background\tstat +\tstat -\n'
            '200.0\t225.0\t0.0\t0.0\t0.0\n'
            '225.0\t250.0\t0.95\t0.41\t-0.51\n'
            '250.0\t275.0\t0.9\t0.41\t-0.26\n'
            '275.0\t300.0\t0.42\t0.12\t-0.19\n'
            '300.0\t325.0\t0.34\t0.16\t-0.15\n'
            '325.0\t350.0\t0.07\t0.19\t-0.16\n'
            '350.0\t375.0\t0.68\t0.56\t-0.55\n'
            '375.0\t400.0\t0.17\t0.1\t-0.15\n'
            '400.0\t425.0\t0.24\t0.11\t-0.1\n'
            '425.0\t450.0\t0.01\t0.08\t0.08\n'
            '450.0\t475.0\t0.3\t0.33\t0.33\n'
            '475.0\t500.0\t0.16\t0.17\t-0.14\n'
            '\n'
            '#: SQRT(S)\t\t8000.0\n'
            '#: EVENTS\t\t25\n'
            'ETMISS IN GEV LOW\tETMISS IN GEV HIGH\tGGM 700 200 1.5\n'
            '200.0\t225.0\t0.0\n'
            '225.0\t250.0\t6.46\n'
            '250.0\t275.0\t6.82\n'
            '275.0\t300.0\t2.82\n'
            '300.0\t325.0\t2.41\n'
            '325.0\t350.0\t3.11\n'
            '350.0\t375.0\t0.7\n'
            '375.0\t400.0\t0.9\n'
            '400.0\t425.0\t0.69\n'
            '425.0\t450.0\t0.72\n'
            '450.0\t475.0\t0.0\n'
            '475.0\t500.0\t0.93\n'
            '\n'
            '#: SQRT(S)\t\t8000.0\n'
            '#: EVENTS\t\t25\n'
            'ETMISS IN GEV LOW\tETMISS IN GEV HIGH\tGGM 900 600 1.5\n'
            '200.0\t225.0\t0.0\n'
            '225.0\t250.0\t0.97\n'
            '250.0\t275.0\t1.07\n'
            '275.0\t300.0\t1.17\n'
            '300.0\t325.0\t1.05\n'
            '325.0\t350.0\t1.08\n'
            '350.0\t375.0\t1.13\n'
            '375.0\t400.0\t1.2\n'
            '400.0\t425.0\t1.01\n'
            '425.0\t450.0\t0.94\n'
            '450.0\t475.0\t0.88\n'
            '475.0\t500.0\t4.59\n'
            '\n'
        )

    def test_csvwriter_options(self):
        csv_content = convert(self.submission_filepath, options={'input_format': 'yaml',
                                                                 'output_format': 'csv',
                                                                 'table': 'Table 1',
                                                                 'pack': True})

        self.assertEqual(self.table_csv, csv_content)

    def test_2_qualifiers_2_iv_pack(self):
        csv_content = convert(self.submission_filepath, options={'input_format': 'yaml',
                                                                 'output_format': 'csv',
                                                                 'table': 'Table 9',
                                                                 'pack': True})

        self.assertEqual(self.table_2_csv, csv_content)

    def test_multiple_tables_pack(self):
        convert(self.submission_filepath, self.current_tmp, options={'input_format': 'yaml',
                                                                     'output_format': 'csv',
                                                                     'pack': True})

        with open(os.path.join(self.current_tmp, 'Table 1.csv'), 'r') as f:
            self.assertEqual(self.table_csv, f.read())

        with open(os.path.join(self.current_tmp, 'Table 9.csv'), 'r') as f:
            self.assertEqual(self.table_2_csv, f.read())

    def test_2_qualifiers_2_iv_unpack(self):
        csv_content = convert(self.submission_filepath, options={'input_format': 'yaml',
                                                         'output_format': 'csv',
                                                         'table': 'Table 9',
                                                         'pack': False})
        self.assertEqual(self.table_2_packed_csv, csv_content)




import abc
import copy
import csv
import os
from hepdata_converter.common import OptionInitMixin, Option
from hepdata_converter.writers import Writer


class ArrayWriter(Writer):
    __metaclass__  = abc.ABCMeta
    options = {
        'table': Option('table', 't', required=False, variable_mapping='table_id', default=None,
                        help=('Specifies which table should be exported, if not specified all tables will be exported '
                              '(in this case output must be a directory, not a file)')),
    }

    def __init__(self, *args, **kwargs):
        super(ArrayWriter, self).__init__(single_file_output=True, *args, **kwargs)
        OptionInitMixin.__init__(self, options=kwargs)
        self.tables = []
        self.extension = None

    @abc.abstractmethod
    def _write_table(self, data_out, table):
        pass

    def write(self, data_in, data_out, *args, **kwargs):
        """

        :param data_in:
        :type data_in: hepconverter.parsers.ParsedData
        :param data_out: filelike object
        :type data_out: file
        :param args:
        :param kwargs:
        """
        # get table to work on
        if self.table_id is not None:
            if isinstance(self.table_id, int):
                self.tables.append(data_in.get_table(id=self.table_id))
            else:
                try:
                    tab = data_in.get_table(file=self.table_id)
                except IndexError:
                    tab = data_in.get_table(name=self.table_id)

                self.tables.append(tab)
        else:
            self.tables = data_in.tables

        file_emulation = False
        outputs = []

        if isinstance(data_out, str) or isinstance(data_out, unicode):
            file_emulation = True
            if len(self.tables) == 1:
                f = open(data_out, 'w')
                outputs.append(f)
            else:
                for table in self.tables:
                    outputs.append(open(os.path.join(data_out, table.name+'.'+self.extension), 'w'))
        # multiple tables - require directory
        elif len(self.tables) > 1 and not (isinstance(data_out, str) or isinstance(data_out, unicode)):
            raise ValueError("Multiple tables, output must be a directory")
        else:
            outputs.append(data_out)

        for i in xrange(len(self.tables)):
            data_out = outputs[i]
            table = self.tables[i]

            self._write_table(data_out, table)

            if file_emulation:
                data_out.close()

    @classmethod
    def _extract_independent_variables(cls, table, headers, data, qualifiers_marks):
        for independent_variable in table.independent_variables:
            headers.append(independent_variable['header']['name'] + ' IN %s' % independent_variable['header']['units'])
            x_data_low = []
            x_data_high = []
            for value in independent_variable['values']:

                if 'high' in value and 'low' in value:
                    x_data_low.append(value['low'])
                    x_data_high.append(value['high'])
                else:
                    x_data_low.append(value['value'])
                    x_data_high.append(value['value'])

            data.append(x_data_low)
            if x_data_high != x_data_low:
                data.append(x_data_high)
                header = headers[-1]
                headers[-1] = header + ' LOW'
                headers.append(header + ' HIGH')
                qualifiers_marks.append(False)

    @classmethod
    def _parse_dependent_variable(cls, dependent_variable, headers, qualifiers, qualifiers_marks, data):
        units = ''
        if 'units' in dependent_variable['header']:
            units = ' IN %s' % dependent_variable['header']['units']
        headers.append(dependent_variable['header']['name'] + units)

        qualifiers_marks.append(True)

        # peek at first value and create empty lists
        y_order = []
        y_data = {'values': []}
        y_order.append(y_data['values'])
        for error in dependent_variable['values'][0].get('errors', []):
            headers.append(error.get('label', 'stat') + ' +')
            qualifiers_marks.append(False)
            headers.append(error.get('label', 'stat') + ' -')
            qualifiers_marks.append(False)

            plus = []
            y_data[error.get('label', 'stat')+'_plus'] = plus
            y_order.append(plus)
            minus = []
            y_data[error.get('label', 'stat')+'_minus'] = minus
            y_order.append(minus)

        for value in dependent_variable['values']:
            y_data['values'].append(value['value'])
            for i in xrange(len(value.get('errors', []))):
                error = value['errors'][i]

                if 'symerror' in error:
                    error_plus = error['symerror']
                    error_minus = error['symerror']
                else:
                    error_plus = error['asymerror']['plus']
                    error_minus = error['asymerror']['minus']

                y_data[error.get('label', 'stat')+'_plus'].append(error_plus)
                y_data[error.get('label', 'stat')+'_minus'].append(error_minus)

        for entry in y_order:
            data.append(entry)

        for qualifier in dependent_variable.get('qualifiers', []):
            if qualifier['name'] not in qualifiers:
                qualifiers[qualifier['name']] = []
            qualifiers[qualifier['name']].append(qualifier['value'])


import unittest
from hepdata_converter.common import OptionInitMixin, Option


class OptionInitMixinTestSuite(unittest.TestCase):
    def test_dir(self):
        class TestClass(OptionInitMixin):
            options = {
                'testoption': Option('testoption')
            }

            def __init__(self, *args, **kwargs):
                OptionInitMixin.__init__(self, options=kwargs)

        test_value = 1
        a = TestClass(testoption=test_value)
        self.assertEqual(a.testoption, test_value)
        self.assertIn('testoption', dir(a))




"""Here data which in other cases would make unit tests files hard to read is stored

"""


OLD_HEPDATA_LONG =  """# Sample input file for new HepData submissions.
# Annotated version of http://hepdata.cedar.ac.uk/view/ins1203852/input
# See resulting HepData record at http://hepdata.cedar.ac.uk/view/ins1203852
# Any text after # in a line is treated as a comment and will be ignored.
# Keywords are indicated by an asterisk (*) at the start of a line.

# Test data input at http://hepdata.cedar.ac.uk/input
# by clicking on "Browse..." to select your input file, followed by
# "Upload" and "Process", then click "Display" for the first 10 tables,
# or "All" for the whole record.
# See http://hepdata.cedar.ac.uk/submittingdata for further information.

# Give surname (in capital letters) of first author of associated paper.
*author: AAD

# Give references ending in a colon followed by the year.
# Omit the journal reference if the paper is not yet published.
*reference: ARXIV:1211.6096 : 2012 # arXiv number
*reference: CERN-PH-EP-2012-318 : 2012 # preprint number
*reference: JHEP 1303,128 (2013) : 2013 # journal reference

# Give web link to auxiliary information if available.
# The line must also end with a colon followed by the year.
*reference: http://atlas.web.cern.ch/Atlas/GROUPS/PHYSICS/PAPERS/STDM-2012-02/ : 2012

# Give DOI of journal publication if available.
*doi: 10.1007/JHEP03(2013)128

# Give date and person who encoded the record
# (could include collaboration, e-mail address etc.).
# Please give the full name of the encoder and not
# just their initials (with collaboration in brackets).
*status: Encoded 27 FEB 2014 by Graeme Watt
# Can add other entries if record is later modified.
# The first word "Encoded" or "Modified" will be stored.
*status: Modified 30 APR 2014 by Graeme Watt
# If using the automated submission system, could add name
# of Administrator as well as Encoder, e.g. "Checked by ...".
# The *status information is printed at the bottom of each HTML record.

# Give "Laboratory-Accelerator-Experiment" and Detector.
*experiment: CERN-LHC-ATLAS
*detector: ATLAS

# SPIRES (http://www.slac.stanford.edu/spires/) record number.
# Will not be allocated for new papers: superseded by INSPIRE.
*spiresId: 9863591

# INSPIRE (http://inspirehep.net/) record number.
*inspireId: 1203852

# CERN Document Server (http://cds.cern.ch/) record number.
*cdsId: 1496097

# Internal Durham record number (will be assigned after submission).
*durhamId: 6150

# Title of associated paper.
*title: Measurement of ZZ production in pp collisions at sqrt(s) = 7 TeV and limits on anomalous ZZZ and ZZgamma couplings with the ATLAS detector

# Comments describing the data as a whole.  Start with "Laboratory-Accelerator".
# Remainder is a plain-text summary of the presented measurements.  This is
# generally close to the paper abstract, but with more focus on the measurement.
# For example, information on data interpretation and comparison to theory can
# be removed (unless this information is to appear in the HepData record).
# Additional information concerning the whole data set can appear here.
# Line breaks are preserved in the HTML display.  A full stop is automatically
# inserted at the end of the comment, so it should be omitted here.

*comment: CERN-LHC.  Measurements of the cross section for ZZ production using the 4l and 2l2nu decay channels in proton-proton collisions at a centre-of-mass energy of 7 TeV with 4.6 fb^-1 of data collected in 2011.  The final states used are 4 electrons, 4 muons, 2 electrons and 2 muons, 2 electrons and missing transverse momentum, and 2 muons and missing transverse momentum (MET).

The cross section values reported in the tables should be multiplied by a factor of 1.0141 to take into account the updated value of the integrated luminosity for the ATLAS 2011 data taking period.  The uncertainty on the global normalisation ("Lumi") remains at 1.8%.  See Eur.Phys.J. C73 (2013) 2518 for more details.

The 4l channel fiducial region is defined as:
- 4e, 4mu or 2e2mu
- Ambiguities in pairing are resolved by choosing the combination that results in the smaller value of the sum |mll - mZ| for the two pairs, where mll is the mass of the dilepton system.
- ptLepton > 7 GeV (at least one with ptLepton > 20 (25) GeV for muons (electrons))
- |etaLepton| < 3.16
- At least one lepton pair is required to have invariant mass between 66 and 116 GeV. If the second pair also satisfies this, the event is ZZ, otherwise if the second pair satisfies mll > 20 GeV it is ZZ*.
- min(DeltaR(l,l)) > 0.2.

The 2l2nu channel fiducial region is defined as:
- 2e+MET or 2mu+MET
- ptLepton > 20 GeV
- |etaLepton| < 2.5
- mll must be between 76 and 106 GeV
- -MET*cos(phi_METZ)>75 GeV, where phi_METZ is the angle between the Z and the MET
- |MET - pTZ| / pTZ < 0.4, where pTZ is the transverse momentum of the dilepton system
- No events with a jet for which ptJet > 25 GeV and |etaJet| < 4.5
- No events with a third lepton for which ptLepton > 10 GeV
- min(DeltaR(l,l)) > 0.3


# Note that much of the metadata above can be obtained by entering the
# Inspire ID or arXiv number for the paper in the text box adjacent to
# the "Get started" button.

# Remainder of file provides information on each table.
# Note that much of the HepData notation is historical and originates from
# 1970s guidelines for use with the Berkeley Database Management System (BDMS):
#   http://hepdata.cedar.ac.uk/resource/EncodingManual.pdf
# You can look at existing HepData records to get some idea of the format.
# LaTeX mathematical expressions are now supported via MathJax.

# Use "*dataset:" to indicate the start of a table.
*dataset:

# Give location of relevant data in paper (e.g. "Figure 8A", "Table 3" etc.).
*location: Page 17 of preprint

# Comments on this particular table.  This is generally close to the caption of
# the associated figure/table in the paper.  A full stop is automatically
# inserted at the end of the dscomment, so it should be omitted here.
*dscomment: The measured fiducial cross sections.  The first systematic uncertainty is the combined systematic uncertainty excluding luminosity, the second is the luminosity

# The main process under study, used by the online search facility.
# The *reackey can be omitted if there is not a specific process.
# Multiple *reackey processes can be given on separate lines.
# Do not attempt to give multiple processes on the same line.
# Omit the decay products or give them on a separate line.
*reackey: P P --> Z0 Z0 X

# The particular type of measurement or observable, used by the online search.
# Use "SIG" for a cross section, "DSIG/DPT" for a differential cross section,
# "N" for number of events, etc.  See:  http://hepdata.cedar.ac.uk/browse
# Avoid adding new values not already in the database, found by browsing
# or searching for "obs OBSKEY" on the main HepData webpage.  If in doubt,
# just omit the *obskey statement completely.  The *obskey value must consist
# of only a single word, but multiple values can be given on separate lines.
*obskey: SIG

# There are usually multiple lines beginning with "*qual:" giving the reaction
# (as below), the centre-of-mass energy, and important kinematic cuts.
# Here, the three columns of the table correspond to different final states.
# Decay products of the final-state particles are indicated by angular brackets.
*qual: RE : P P --> Z0 < LEPTON+ LEPTON- > Z0 < LEPTON+ LEPTON- > X : P P --> Z0 < LEPTON+ LEPTON- > Z0* < LEPTON+ LEPTON- > X : P P --> Z0 < LEPTON+ LEPTON- > Z0 < NU NUBAR > X

# Headers for the x columns separated by colons ":" (only one in this case).
*xheader: SQRT(S) IN GEV

# Headers for the y columns separated by ":" (here, header spans three columns).
*yheader: SIG(fiducial) IN FB

# Specify number of columns separated by ":" (here, 1 "x" and 3 "y" columns).
*data: x : y : y : y

# Now give the numerical values for the data, separating columns by a
# semicolon (";").  Non-existent entries can be indicated by dashes "-".
# Numerical values should be given with an appropriate and not excessive number
# of significant figures (in general, a maximum of 4), avoiding giving more
# decimal places for the errors than the central values.
# After the central value, give the statistical uncertainties followed by the
# (possibly multiple) systematic uncertainties in brackets as shown below.
# A label can be given to distinguish multiple systematic uncertainties, as
# shown below for the "lumi" case, by a colon after the number, then the label.

 7000; 25.4 +3.3,-3.0 (DSYS=+1.2,-1.0,DSYS=1.0:lumi); 29.8 +3.8,-3.5 (DSYS=+1.7,-1.5,DSYS=1.2:lumi); 12.7 +3.1,-2.9 (DSYS=1.7,DSYS=0.5:lumi);

# Use "*dataend:" to indicate the end of a table (optional, can be omitted,
# provided that "*E" is used at the end of the input file).
*dataend:


# Similarly for Table 2 ...
*dataset:
*location: Page 20 of preprint
*dscomment: The measured total cross sections.  The first systematic uncertainty is the combined systematic uncertainty excluding luminosity, the second is the luminosity
*reackey: P P --> Z0 Z0 X
*obskey: SIG
*qual: RE : P P --> Z0 Z0 X
*yheader: SIG(total) IN FB
*xheader: SQRT(S) IN GEV
*data: x : y
 7000; 6.7 +- 0.7 (DSYS=+0.4,-0.3,DSYS=0.3:lumi);
*dataend:


# Similarly for Table 3 ...
# Note units of GEV not GEV/C (i.e. C = 1) for transverse momentum PT.
# The x values are given as the bin centre (not necessarily equal to midpoint)
# followed by the lower and upper limits of the bin.  In the common case that
# the bin centre is equal to the midpoint, then it can be omitted,
# i.e. just " 0 TO 60;" instead of " 30 (BIN=0 TO 60);".  Note that the
# formatter currently appends an unnecessary ".0" to some integers, as below.
*dataset:
*location: Figure 8A
*dscomment: Normalized ZZ fiducial cross section (multiplied by 10^6 for readability) in bins of the leading reconstructed dilepton pT for the 4 lepton channel.  The first systematic uncertainty is detector systematics, the second is background systematic uncertainties
*reackey: P P --> Z0 Z0 X
*obskey: DSIG/DPT
*qual: RE : P P --> Z0 < LEPTON+ LEPTON- > Z0 < LEPTON+ LEPTON- > X
*qual: SQRT(S) IN GEV : 7000.0
*yheader: 10**6 * 1/SIG(fiducial) * D(SIG(fiducial))/DPT IN GEV**-1
*xheader: Leading dilepton PT IN GEV
*data: x : y
 0.0 TO 60.0; 7000 +- 1100.0 (DSYS=79.0:detector,DSYS=15.0:background);
 60.0 TO 100.0; 9800 +- 1600.0 (DSYS=75.0:detector,DSYS=15.0:background);
 100.0 TO 200.0; 1600 +- 490.0 (DSYS=41.0:detector,DSYS=2.0:background);
 200.0 TO 600.0; 80 +- 60.0 (DSYS=2.0:detector,DSYS=0.0:background);
*dataend:

# Similarly for Table 4 ...
*dataset:
*location: Figure 8B
*dscomment: Normalized ZZ fiducial cross section (multiplied by 10^6 for readability) in bins of the dilepton pT for the 2l2nu channel.  The first systematic uncertainty is detector systematics, the second is background systematic uncertainties
*reackey: P P --> Z0 Z0 X
*obskey: DSIG/DPT
*qual: RE : P P --> Z0 < LEPTON+ LEPTON- > Z0 < NU NUBAR > X
*qual: SQRT(S) IN GEV : 7000.0
*yheader: 10**6 * 1/SIG(fiducial) * D(SIG(fiducial))/DPT IN GEV**-1
*xheader: Leading dilepton PT IN GEV
*data: x : y
 50.0 TO 90.0; 9930 +- 3340.0 (DSYS=80.0:detector,DSYS=740.0:background);
 90.0 TO 130.0; 8280 +- 3210.0 (DSYS=200.0:detector,DSYS=260.0:background);
 130.0 TO 200.0; 3900 +- 1490.0 (DSYS=120.0:detector,DSYS=390.0:background);
*dataend:

# Similarly for Table 5 ...
*dataset:
*location: Figure 9A
*dscomment: Normalized ZZ fiducial cross section (multiplied by 10^6 for readability) in bins of deltaPhi between the two leptons of the leading dileptons for the 4l channel.  The first systematic uncertainty is detector systematics, the second is background systematic uncertainties.
UPDATE (30 APR 2014): extra significant digit added for first bin
*reackey: P P --> Z0 Z0 X
*obskey: DSIG/DPHI
*qual: RE : P P --> Z0 < LEPTON+ LEPTON- > Z0 < LEPTON+ LEPTON- > X
*qual: SQRT(S) IN GEV : 7000.0
*yheader: 10**6 * 1/SIG(fiducial) * D(SIG(fiducial))/DDELTA(PHI(LEPTON+,LEPTON-))
*xheader: Leading dilepton DELTA(PHI(LEPTON+,LEPTON-))
*data: x : y
 0.0 TO 0.5; 130000 +- 69000.0 (DSYS=6600.0:detector,DSYS=10.0:background);
 0.5 TO 1.0; 280000 +- 100000.0 (DSYS=9000.0:detector,DSYS=400.0:background);
 1.0 TO 1.7; 260000 +- 80000.0 (DSYS=10000.0:detector,DSYS=300.0:background);
 1.7 TO 3.14159; 420000 +- 50000.0 (DSYS=2000.0:detector,DSYS=300.0:background);
*dataend:

# Similarly for Table 6 ...
*dataset:
*location: Figure 9B
*dscomment: Normalized ZZ fiducial cross section (multiplied by 10^6 for readability) in bins of deltaPhi between the two leptons for the 2l2nu channel.  The first systematic uncertainty is detector systematics, the second is background systematic uncertainties
*reackey: P P --> Z0 Z0 X
*obskey: DSIG/DPHI
*qual: RE : P P --> Z0 < LEPTON+ LEPTON- > Z0 < NU NUBAR > X
*qual: SQRT(S) IN GEV : 7000.0
*yheader: 10**6 * 1/SIG(fiducial) * D(SIG(fiducial))/DDELTA(PHI(LEPTON+,LEPTON-))
*xheader: Leading dilepton DELTA(PHI(LEPTON+,LEPTON-))
*data: x : y
 0.0 TO 0.5; 346000 +- 158000.0 (DSYS=4000.0:detector,DSYS=19000.0:background);
 0.5 TO 1.7; 569000 +- 91000.0 (DSYS=4000.0:detector,DSYS=71000.0:background);
 1.7 TO 3.14159; 100000 +- 70000.0 (DSYS=2000.0:detector,DSYS=58000.0:background);
*dataend:

# Similarly for Table 7 ...
*dataset:
*location: Figure 10A
*dscomment: Normalized ZZ fiducial cross section (multiplied by 10^6 for readability) in bins of the mass of the ZZ system for the 4l channel.  The first systematic uncertainty is detector systematics, the second is background systematic uncertainties
*reackey: P P --> Z0 Z0 X
*obskey: DSIG/DM
*qual: RE : P P --> Z0 < LEPTON+ LEPTON- > Z0 < LEPTON+ LEPTON- > X
*qual: SQRT(S) IN GEV : 7000.0
*yheader: 10**6 * 1/SIG(fiducial) * D(SIG(fiducial))/DM(ZZ)
*xheader: M(ZZ) IN GEV
*data: x : y
 0.0 TO 240.0; 2200 +- 300.0 (DSYS=40.0:detector,DSYS=2.0:background);
 240.0 TO 300.0; 4500 +- 1000.0 (DSYS=100.0:detector,DSYS=5.0:background);
 300.0 TO 400.0; 1000 +- 400.0 (DSYS=20.0:detector,DSYS=2.0:background);
 400.0 TO 800.0; 280 +- 100.0 (DSYS=10.0:detector,DSYS=1.0:background);
*dataend:

# Similarly for Table 8 ...
*dataset:
*location: Figure 10B
*dscomment: Normalized ZZ fiducial cross section (multiplied by 10^6 for readability) in bins of the transverse mass of the ZZ system for the 2l2nu channel.  The first systematic uncertainty is detector systematics, the second is background systematic uncertainties
*reackey: P P --> Z0 Z0 X
*obskey: DSIG/DM
*qual: RE : P P --> Z0 < LEPTON+ LEPTON- > Z0 < NU NUBAR > X
*qual: SQRT(S) IN GEV : 7000.0
*yheader: 10**6 * 1/SIG(fiducial) * D(SIG(fiducial))/DMT(ZZ)
*xheader: MT(ZZ) IN GEV
*data: x : y
 220.0 TO 250.0; 10500 +- 4400.0 (DSYS=300.0:detector,DSYS=1900.0:background);
 250.0 TO 300.0; 6320 +- 2630.0 (DSYS=230.0:detector,DSYS=280.0:background);
 300.0 TO 400.0; 3680 +- 1210.0 (DSYS=60.0:detector,DSYS=480.0:background);
*dataend:


# Use "*E" to denote the end of the input file (optional, can be omitted,
# provided that "*dataend:" is used above).

*E"""


import unittest
import os
import tempfile
import shutil
import time
from hepdata_converter import convert
from hepdata_converter.testsuite.test_writer import WriterTestSuite


class ArrayWriterTestSuite(WriterTestSuite):
    def test_select_table(self):
        csv_content = convert(self.submission_filepath, options={'input_format': 'yaml',
                                                                 'output_format': 'csv',
                                                                 'table': os.path.join(os.path.dirname(self.submission_filepath), 'data1.yaml')})

        csv_content = convert(self.submission_filepath, options={'input_format': 'yaml',
                                                                 'output_format': 'csv',
                                                                 'table': 'Table 1'})

        csv_content = convert(self.submission_filepath, options={'input_format': 'yaml',
                                                                 'output_format': 'csv',
                                                                 'table': 0})



import StringIO
import argparse
import os
import sys
from hepdata_converter.parsers import Parser
from hepdata_converter.writers import Writer


def convert(input, output=None, options={}):
    """Converts arbitrary supported data format to the new HEPData YAML format, and writes the output files
    to the output_dir directory

    :param input: input, depending on the choosen input datatype it may be, filepath, filetype object, directory, etc
    :param output: output directory to which converted YAML files will be written
    :param input_format: format of the input data, must be a string containing name of the input parser class
    :type input_format: str
    :param options: additional options used for conversion (depends on the choosen input format)
    :type options: dict
    :raise ValueError: raised if no input_format is specified
    """

    if 'input_format' not in options and 'output_format' not in options:
        raise ValueError("no input_format and output_format specified!")

    input_format = options.get('input_format', 'yaml')
    output_format = options.get('output_format', 'yaml')

    # nothing to do
    if input_format == output_format:
        return

    parser = Parser.get_concrete_class(input_format)(**options)
    writer = Writer.get_concrete_class(output_format)(**options)

    if not output and not writer.single_file_output:
        raise ValueError("this output_format requires specifing 'output' argument")

    # if no output was specified create proxy output to which writer can insert data
    _output = output
    if not _output:
        _output = StringIO.StringIO()

    writer.write(parser.parse(input), _output)

    # if no output was specified return output
    if not output:
        return _output.getvalue()


def main(arguments=sys.argv):
    parser = argparse.ArgumentParser(prog='hepdata-converter', description="CLI tools for converting between HEP data formats", add_help=True)
    parser.add_argument("--input-format", '-i', required=True)
    parser.add_argument("--output-format", '-o', required=True)
    parser.add_argument("input")
    parser.add_argument("output")

    program_args = vars(parser.parse_known_args(arguments)[0])

    input_format = program_args['input_format']
    output_format = program_args['output_format']

    Parser.get_concrete_class(input_format).register_cli_options(parser)
    Writer.get_concrete_class(output_format).register_cli_options(parser)

    # reparse arguments, now with added options from concrete parsers / writers
    program_args = vars(parser.parse_args(arguments))

    try:
        convert(program_args['input'], program_args['output'], program_args)
    except ValueError as e:
        sys.exit("Options error: %s" % str(e))


import tempfile
import unittest
import StringIO
import time
import os
import shutil
from hepdata_converter.parsers.oldhepdata import OldHEPData
from testdata import OLD_HEPDATA_LONG


class OldHEPDataTestSuite(unittest.TestCase):
    """Test suite for OldHEPData parser (and YAML writer).
    These are mostly class tests, and unit tests
    """

    def tearDown(self):
        shutil.rmtree(self.tmp_dir_name)

    def setUp(self):
        self.tmp_dir_name = os.path.join(tempfile.gettempdir(), str(int(time.time())))
        os.mkdir(self.tmp_dir_name)

    def test_parse_submission(self):
        parser = OldHEPData()
        parsed_data = parser.parse(StringIO.StringIO(OLD_HEPDATA_LONG))

        # test parsing of independent_variables
        self.assertEqual(parsed_data.tables[0].data['independent_variables'][0], {'values': ['7000'], 'header': {'name': 'SQRT(S)', 'units': 'GEV'}})

        # test parsing of dependent_variables
        self.assertEqual(parsed_data.tables[0].data['dependent_variables'], [
            {'header': {'units': 'FB', 'name': 'SIG(fiducial)'}, 'qualifiers': [{'name': 'RE', 'value': 'P P --> Z0 < LEPTON+ LEPTON- > Z0 < LEPTON+ LEPTON- > X'}], 'values': [
                {'value': '25.4',
                 'errors': [
                    {'asymerror': {'plus': '3.3', 'minus': '-3.0'}, 'label': 'stat'},
                    {'asymerror': {'minus': '-1.0', 'plus': '1.2'}, 'label': 'sys'},
                    {'symerror': '1.0', 'label': 'sys,lumi'},
                 ]
                }
            ]},
            {'header': {'units': 'FB', 'name': 'SIG(fiducial)'}, 'qualifiers': [{'name': 'RE', 'value': 'P P --> Z0 < LEPTON+ LEPTON- > Z0* < LEPTON+ LEPTON- > X'}], 'values': [
                {
                    'value': '29.8',
                    'errors': [{'asymerror': {'plus': '3.8', 'minus': '-3.5'}, 'label': 'stat'},
                               {'asymerror': {'plus': '1.7', 'minus': '-1.5'}, 'label': 'sys'},
                               {'symerror': '1.2', 'label': 'sys,lumi'}]
                }
            ]},
            {'header': {'units': 'FB', 'name': 'SIG(fiducial)'}, 'qualifiers': [{'name': 'RE', 'value': 'P P --> Z0 < LEPTON+ LEPTON- > Z0 < NU NUBAR > X'}], 'values': [
                {
                    'value': '12.7',
                    'errors': [
                        {'asymerror': {'plus': '3.1', 'minus': '-2.9'}, 'label': 'stat'},
                        {'symerror': '1.7', 'label': 'sys'},
                        {'symerror': '0.5', 'label': 'sys,lumi'}
                    ]
                }
            ]
            }])


from abc import ABCMeta
from string import lower


class Option(object):
    def __init__(self, name, shortversion=None, type=str, default=None, variable_mapping=None, required=True, help='', auto_help=True):
        self.name = name
        self.help = help
        self.required = required
        self.auto_help = auto_help
        self.shortversion = shortversion
        self.variable_mapping = variable_mapping or self.name
        self.default = default
        self.type = type
        if self.type is bool:
            self.action = 'store_const'
        else:
            self.action = 'store'

    def attach_to_parser(self, parser):
        args = ['--'+self.name]
        if self.shortversion:
            args.append('-'+self.shortversion)

        kwargs = {
            'default': self.default,
            'help': self.help,
            'required': self.required,
            'action': self.action
        }
        if self.type == bool:
            kwargs['const'] = True

        parser.add_argument(*args, **kwargs)



class OptionInitMixin(object):
    """This mixin requires a class to have specified options dictionary as
    class variable

    """
    def __init__(self, options):
        self.options_values = {}
        for option in options:
            if option in self.__class__.options:
                self.options_values[self.__class__.options[option].variable_mapping] = options[option]

        # add default values
        for key, option in self.__class__.options.items():
            if key not in options:
                self.options_values[self.__class__.options[key].variable_mapping] = option.default

    @classmethod
    def register_cli_options(cls, parser):
        for option in cls.options:
            cls.options[option].attach_to_parser(parser)

    def __getattr__(self, attr_name):
        if attr_name in self.options_values:
            return self.options_values[attr_name]
        else:
            raise AttributeError("'%s' object has no attribute '%s'" % (self.__class__.__name__, attr_name))

    def __dir__(self):
        _dir = dir(super(OptionInitMixin, self))
        _dir += [option.variable_mapping for key, option in self.__class__.options.items()]
        return _dir


class GetConcreteSubclassMixin(object):
    @classmethod
    def get_concrete_class(cls, class_name):
        """This method provides easier access to all writers inheriting Writer class

        :param class_name: name of the parser (name of the parser class which should be used)
        :type class_name: str
        :return: Writer subclass specified by parser_name
        :rtype: Writer subclass
        :raise ValueError:
        """
        def recurrent_class_lookup(cls):
            for cls in cls.__subclasses__():
                if lower(cls.__name__) == lower(class_name):
                    return cls
                elif len(cls.__subclasses__()) > 0:
                    r = recurrent_class_lookup(cls)
                    if r is not None:
                        return r
            return None

        cls = recurrent_class_lookup(cls)
        if cls:
            return cls
        else:
            raise ValueError("'class_name '%s' is invalid" % class_name)



import os
import hepdata_converter
import unittest
from hepdata_converter.testsuite.test_writer import WriterTestSuite


class CLIToolsTestSuite(WriterTestSuite):
    def test_wrong_call(self):
        self.assertRaises(SystemExit, hepdata_converter.main, [])

    def test_convert_yaml2csv(self):
        hepdata_converter.main(['--input-format', 'yaml', '--output-format', 'csv',
                                '--table', 'Table 1', '--pack',
                                self.current_tmp, os.path.join(self.current_tmp, 'output.csv')])
        self.table_csv = (
            '#: name: Table 1\n'
            '#: description: The measured fiducial cross sections. The first systematic uncertainty is the combined systematic uncertainty excluding luminosity, the second is the luminosity\n'
            '#: data_file: data1.yaml\n'
            '#: keyword reactions: P P --> Z0 Z0 X\n'
            '#: keyword observables: SIG\n'
            '#: keyword energies: 7000\n'
            '#: RE\tP P --> Z0 < LEPTON+ LEPTON- > Z0 < LEPTON+ LEPTON- > X\n'
            'SQRT(S) IN GEV\tSIG(fiducial) IN FB\tstat +\tstat -\tsys +\tsys -\tsys,lumi +\tsys,lumi -\n'
            '7000\t25.4\t3.3\t-3.0\t1\t-1.2\t1\t-1\n'
            '8000\t29.8\t3.8\t-3.5\t1.7\t-1.5\t1.2\t1.2\n'
            '9000\t12.7\t3.1\t-2.9\t1.7\t1.7\t0.5\t0.5\n'
        )

        with open(os.path.join(self.current_tmp, 'output.csv')) as f:
            self.assertEqual(self.table_csv, f.read())

    def test_convert_yaml2yoda(self):
        hepdata_converter.main(['--input-format', 'yaml', '--output-format', 'csv',
                                '--table', 'Table 1',
                                self.current_tmp, os.path.join(self.current_tmp, 'output.csv')])

    def test_help(self):
        self.assertRaises(SystemExit, hepdata_converter.main, ['--help'])


import yaml
from hepdata_converter.writers import Writer
import os


class YAML(Writer):
    def __init__(self, *args, **kwargs):
        super(YAML, self).__init__(single_file_output=False, *args, **kwargs)

    def write(self, data_in, data_out, *args, **kwargs):
        """

        :param data_in:
        :type data_in: hepconverter.parsers.ParsedData
        :param data_out: path of the directory to which yaml files will be written
        :type data_out: str
        :param args:
        :param kwargs:
        """


        tables = data_in.tables
        data = data_in.data

        with open(os.path.join(data_out, 'submission.yaml'), 'w') as submission_file:

            yaml.dump_all(data + [table.metadata for table in tables], submission_file)

        for table in tables:
            with open(os.path.join(data_out, table.data_file), 'w') as table_file:
                yaml.dump(table.data, table_file)



import unittest
import os
import tempfile
import shutil
import time


class WriterTestSuite(unittest.TestCase):
    def tearDown(self):
        shutil.rmtree(self.current_tmp)

    def setUp(self):
        self.submission_data = (
            u"---\n"
            u"---\n"
            u'name: "Table 1"\n'
            u'location: Page 17 of preprint\n'
            u'description: The measured fiducial cross sections. The first systematic uncertainty is the combined systematic uncertainty excluding luminosity, the second is the luminosity\n'
            u'keywords: # used for searching, possibly multiple values for each keyword\n'
            u'  - { name: reactions, values: [P P --> Z0 Z0 X]}\n'
            u'  - { name: observables, values: [SIG]}\n'
            u'  - { name: energies, values: [7000]} # centre-of-mass energy in GeV\n'
            u'data_file: data1.yaml\n'
            u'data_license:\n'
            u'---\n'
            u'name: "Table 9"\n'
            u'description: The observed and expected EmissT distribution in the dielectron SR-Z. The negigible estimated contribution from Z+jets is omitted in these distributions. The last bin contains the overflow.\n'
            u'keywords: # used for searching, possibly multiple values for each keyword\n'
            u'  - { name: energies, values: [8000]} # centre-of-mass energy in GeV\n'
            u'data_file: data9.yaml\n')

        self.table_filename = 'data1.yaml'

        self.table_data = (
            u"---\n"
            u"name: 'data1.yaml' #optional, only required if we have a file with everything in it.\n"
            u"independent_variables:\n"
            u"  - header: {name: SQRT(S), units: GEV}\n"
            u"    values:\n"
            u"      - value: 7000\n"
            u"      - value: 8000\n"
            u"      - value: 9000\n"
            u"dependent_variables:\n"
            u"  - header: {name: SIG(fiducial), units: FB}\n"
            u"    qualifiers:\n"
            u"      - {name: RE, value: P P --> Z0 < LEPTON+ LEPTON- > Z0 < LEPTON+ LEPTON- > X}\n"
            u"    values:\n"
            u"      - value: 25.4\n"
            u"        errors:\n"
            u"          - {asymerror: {plus: 3.3, minus: -3.0}, label: stat}\n"
            u"          - {asymerror: {plus: 1, minus: -1.2}, label: sys}\n"
            u"          - {asymerror: {plus: 1, minus: -1}, label: 'sys,lumi'}\n"
            u"  \n"
            u"      - value: 29.8\n"
            u"        errors:\n"
            u"          - {asymerror: {plus: 3.8, minus: -3.5}, label: stat}\n"
            u"          - {asymerror: {plus: 1.7, minus: -1.5}, label: sys}\n"
            u"          - {symerror: 1.2, label: 'sys,lumi'}\n"
            u"  \n"
            u"      - value: 12.7\n"
            u"        errors:\n"
            u"          - {asymerror:{plus: 3.1, minus: -2.9}, label: stat}\n"
            u"          - {symerror: 1.7, label: sys}\n"
            u"          - {symerror: 0.5, label: 'sys,lumi'}\n")

        self.table_data_2_qual = (
            u"independent_variables:\n"
            u"  - header: {name: 'ETMISS', units: 'GEV'}\n"
            u"    values:\n"
            u"      - {low: 200.0, high: 225.0}\n"
            u"      - {low: 225.0, high: 250.0}\n"
            u"      - {low: 250.0, high: 275.0}\n"
            u"      - {low: 275.0, high: 300.0}\n"
            u"      - {low: 300.0, high: 325.0}\n"
            u"      - {low: 325.0, high: 350.0}\n"
            u"      - {low: 350.0, high: 375.0}\n"
            u"      - {low: 375.0, high: 400.0}\n"
            u"      - {low: 400.0, high: 425.0}\n"
            u"      - {low: 425.0, high: 450.0}\n"
            u"      - {low: 450.0, high: 475.0}\n"
            u"      - {low: 475.0, high: 500.0}\n"
            u"dependent_variables:\n"
            u"  - header: {name: Data}\n"
            u"    qualifiers:\n"
            u"      - {name: SQRT(S), value: '8000.0', units: 'GeV'}\n"
            u"      - {name: EVENTS, value: '25', units: 'GEV'}\n"
            u"    values:\n"
            u"      - value: 0.0\n"
            u"      - value: 6.0\n"
            u"      - value: 1.0\n"
            u"      - value: 1.0\n"
            u"      - value: 1.0\n"
            u"      - value: 2.0\n"
            u"      - value: 1.0\n"
            u"      - value: 1.0\n"
            u"      - value: 0.0\n"
            u"      - value: 1.0\n"
            u"      - value: 0.0\n"
            u"      - value: 2.0\n"
            u"  - header: {name: 'Expected Background'}\n"
            u"    qualifiers:\n"
            u"      - {name: SQRT(S), value: '8000.0', units: 'GeV'}\n"
            u"      - {name: EVENTS, value: '25', units: 'GEV'}\n"
            u"    values:\n"
            u"      - value: 0.0\n"
            u"        errors:\n"
            u"          - {symerror: 0.0}\n"
            u"      - value: 0.95\n"
            u"        errors:\n"
            u"          - {asymerror: {plus: 0.41, minus: -0.51}}\n"
            u"      - value: 0.9\n"
            u"        errors:\n"
            u"          - {asymerror: {plus: 0.41, minus: -0.26}}\n"
            u"      - value: 0.42\n"
            u"        errors:\n"
            u"          - {asymerror: {plus: 0.12, minus: -0.19}}\n"
            u"      - value: 0.34\n"
            u"        errors:\n"
            u"          - {asymerror: {plus: 0.16, minus: -0.15}}\n"
            u"      - value: 0.07\n"
            u"        errors:\n"
            u"          - {asymerror: {plus: 0.19, minus: -0.16}}\n"
            u"      - value: 0.68\n"
            u"        errors:\n"
            u"          - {asymerror: {plus: 0.56, minus: -0.55}}\n"
            u"      - value: 0.17\n"
            u"        errors:\n"
            u"          - {asymerror: {plus: 0.1, minus: -0.15}}\n"
            u"      - value: 0.24\n"
            u"        errors:\n"
            u"          - {asymerror: {plus: 0.11, minus: -0.1}}\n"
            u"      - value: 0.01\n"
            u"        errors:\n"
            u"          - {symerror: 0.08}\n"
            u"      - value: 0.3\n"
            u"        errors:\n"
            u"          - {symerror: 0.33}\n"
            u"      - value: 0.16\n"
            u"        errors:\n"
            u"          - {asymerror: {plus: 0.17, minus: -0.14}}\n"
            u"  - header: {name: 'GGM 700 200 1.5'}\n"
            u"    qualifiers:\n"
            u"      - {name: SQRT(S), value: '8000.0', units: 'GeV'}\n"
            u"      - {name: EVENTS, value: '25', units: 'GEV'}\n"
            u"    values:\n"
            u"      - value: 0.0\n"
            u"      - value: 6.46\n"
            u"      - value: 6.82\n"
            u"      - value: 2.82\n"
            u"      - value: 2.41\n"
            u"      - value: 3.11\n"
            u"      - value: 0.7\n"
            u"      - value: 0.9\n"
            u"      - value: 0.69\n"
            u"      - value: 0.72\n"
            u"      - value: 0.0\n"
            u"      - value: 0.93\n"
            u"  - header: {name: 'GGM 900 600 1.5'}\n"
            u"    qualifiers:\n"
            u"      - {name: SQRT(S), value: '8000.0', units: 'GeV'}\n"
            u"      - {name: EVENTS, value: '25', units: 'GEV'}\n"
            u"    values:\n"
            u"      - value: 0.0\n"
            u"      - value: 0.97\n"
            u"      - value: 1.07\n"
            u"      - value: 1.17\n"
            u"      - value: 1.05\n"
            u"      - value: 1.08\n"
            u"      - value: 1.13\n"
            u"      - value: 1.2\n"
            u"      - value: 1.01\n"
            u"      - value: 0.94\n"
            u"      - value: 0.88\n"
            u"      - value: 4.59\n")

        self.table_filename_2 = 'data9.yaml'

        self.current_tmp = os.path.join(tempfile.gettempdir(), str(int(time.time())))
        os.mkdir(self.current_tmp)

        self.submission_filepath = os.path.join(self.current_tmp, 'submission.yaml')
        with open(self.submission_filepath, 'w') as submission:
            submission.write(self.submission_data)
        with open(os.path.join(self.current_tmp, self.table_filename), 'w') as table:
            table.write(self.table_data)
        with open(os.path.join(self.current_tmp, self.table_filename_2), 'w') as table:
            table.write(self.table_data_2_qual)



# -*- coding: utf-8 -*-
from setuptools import setup

setup(
    name='hepdata-converter',
    version='0.1',
    install_requires=[
        'pyyaml'
    ],
    entry_points={
        'console_scripts': [
            'hepdata-converter = hepdata_converter:main',
        ]
    },
    packages=['hepdata_converter', 'hepdata_converter.parsers', 'hepdata_converter.writers', 'hepdata_converter.testsuite'],
    url='',
    license='',
    author='Michał Szostak',
    author_email='michal.florian.szostak@cern.ch',
    description='Library providing means of conversion between oldhepdata format to new one, and new one to csv / yoda / root etc.'
)


from string import lower
from hepdata_converter.common import OptionInitMixin, Option
from hepdata_converter.parsers import Parser, ParsedData, BadFormat, Table
import StringIO
import copy
import re


class HEPTable(Table):
    """Extension of Table including some place for temporary data needed during conversion
    """
    def __init__(self, index=None, data_file=None, table_name=None):
        super(HEPTable, self).__init__(index, data_file, table_name)

        self.xheaders = []
        self.yheaders = []


class OldHEPData(Parser):
    """Parser for Old HEPData format
    """

    options = {
        'strict': Option('strict', required=True, default=True,
                         help='if specified any additional keywords in OldHEPData file will raise an error'),
        'use_additional_data': Option('use_additional_data', required=True, default=False,
                                      help=('if specified additional data which does not have equivalent in new HEPData format'
                                            ' will be appended to comment section of the output document'))
    }

    def reset(self):
        """Clean any processing data, and prepare object for reuse
        """
        self.current_table = None
        self.tables = []
        self.data = [{}]
        self.additional_data = {}
        self.lines = []
        self.set_state('document')
        self.current_file = None


    def set_state(self, state):
        if state not in OldHEPData.states:
            raise ValueError("unknown state")
        self.current_state = state

    def __init__(self, **kwargs):
        """Constructor
        :param use_additional_data: if set to True additional data which does not have equivalent in new HEPData format
        will be appended to comment section of the output document
        :type use_additional_data: bool

        :param strict: if set to True, any additional keywords (not specified by documentation)
        will raise BadFormat exception during parsing
        :type strict: bool
        """


        # mapping of OLDHEP format's keywords to proper parsing functions
        # all possible keywords are specified here, the ones which aren't used
        # by new data format are either mapped to _pass method which does nothing,
        # or bound to _add_to_comment which adds the specified data to comment section of the
        # output

        # functions are specified by states in which parser may be, it may either be parsing table or document
        # for those two states different sets of directives are permitted
        self.mapping = {
            'document': {
                'reference':  self._parse_reference,
                'dataset':    self._set_table,

                # additional data which have no one to one mapping to new YAML format
                'author':     self._bind_parse_addtional_data('author'),
                'doi':        self._bind_parse_addtional_data('doi'),
                'status':     self._bind_parse_addtional_data('status'),
                'experiment': self._bind_parse_addtional_data('experiment'),
                'detector':   self._bind_parse_addtional_data('detector'),
                'title':      self._bind_parse_addtional_data('title'),

                # add it to record_ids
                'spiresId':   self._bind_parse_record_ids('spires'),
                'inspireId':  self._bind_parse_record_ids('inspire'),
                'cdsId':      self._bind_parse_record_ids('cds'),
                'durhamId':   self._bind_parse_record_ids('durham'),

                'comment':    self._parse_document_comment,
                'E':          self._pass
            },
            'table': {
                'dataend':    self._set_document,
                'location':   self._bind_set_table_metadata('location'),
                'dscomment':  self._bind_set_table_metadata('description', True),
                'reackey':    self._parse_reackey,
                'qual':       self._parse_qual,
                'data':       self._parse_table_data,
                'xheader':    self._parse_xheaders,
                'yheader':    self._parse_yheaders,
                'obskey':     self._parse_obskey,
                'E':          self._pass
            }
        }

        OptionInitMixin.__init__(self, options=kwargs)

    def parse(self, data_in):
        # clean any possible data from previous parsing
        self.reset()

        # for convenience, if provided data_in is string instead of
        # filelike object wrap in into StringIO.StringIO
        if isinstance(data_in, str) or isinstance(data_in, unicode):
            data_in = StringIO.StringIO(data_in)

        self.current_file = data_in

        # parse the file
        while self._parse_line(self.current_file):
            pass

        if self.use_additional_data:
            if len(self.additional_data) > 0:
                self.data[0]['comment'] += 'ADDITIONAL DATA IMPORTED FROM OLD HEPDATA FORMAT: \n'
            for key in self.additional_data:
                for element in self.additional_data[key]:
                    self.data[0]['comment'] += "%s: %s" % (key, element)

        return ParsedData(self.data, self.tables)

    def _parse_line(self, file):
        """Parse single line (or more if particular keyword actually demands it)

        :param file:
        :type file: file
        """

        line = self._strip_comments(file.readline())
        # check if the file ended
        if not line:
            return False

        # line was empty or it was a comment, continue
        if line == '\n':
            return True

        # retrieve keyword and its value
        reg = re.search("^\*(?P<key>[^:#]*)(:(?P<value>.*))?$", line)
        if reg:
            key = reg.group('key').strip()
            value = reg.group('value')

            if key in self.mapping[self.current_state]:
                self.mapping[self.current_state][key](value)
            elif self.strict:
                raise BadFormat("unknown key: *%s" % key)
        else:
            raise BadFormat("line can not be parsed: %s" % line)

        return True

    def _parse_reference(self, data):
        """

        :param data:
        :type data: str
        """
        if 'additional_resources' not in self.data[0]:
            self.data[0]['additional_resources'] = []

        self.data[0]['additional_resources'].append({
            'location': data.split(' : ')[0],
            'description': ''
        })

    def _set_table(self, data):
        """Set current parsing state to 'table',
        create new table object and add it to tables collection
        """
        self.set_state('table')
        self.current_table = HEPTable(index=len(self.tables))
        self.tables.append(self.current_table)
        self.data.append(self.current_table.metadata)

    def _set_document(self, data):
        """Set current parsing state to 'document',
        set current_table to None
        """
        self.set_state('document')
        self.current_table = None

    def _pass(self, data):
        """Empty processing function, map it to keywords if they're not used in the new YAML format
        """
        pass

    def _parse_table_data(self, data):
        """Parse dataset data of the original HEPData format

        :param data: header of the table to be parsed
        :raise ValueError:
        """
        header = data.split(':')

        self.current_table.data_header = header

        for i in range(len(header)):
            header[i] = header[i].strip()

        x_count = header.count('x')
        y_count = header.count('y')

        # use deepcopy to avoid references in yaml... may not be required, and will very probably be refactored
        # TODO - is this aproperiate behavior, or are references in YAML files acceptable, they are certainly less human readable
        self.current_table.data = {'independent_variables': [{'header': self.current_table.xheaders[i] if i < len(self.current_table.xheaders) else copy.deepcopy(self.current_table.xheaders[-1]),
                                                              'values': []} for i in range(x_count)],
                                      'dependent_variables': [{'header': self.current_table.yheaders[i] if i < len(self.current_table.yheaders) else copy.deepcopy(self.current_table.yheaders[-1]),
                                                               'qualifiers': [self.current_table.qualifiers[j][i] for j in range(len(self.current_table.qualifiers)) ],
                                                               'values': []} for i in range(y_count)]}

        xy_mapping = []

        current_x_count = 0
        current_y_count = 0

        for i in range(len(header)):
            if header[i] == 'x':
                xy_mapping.append(current_x_count)
                current_x_count += 1
            if header[i] == 'y':
                xy_mapping.append(current_y_count)
                current_y_count += 1

        last_index = self.current_file.pos
        line = self._strip_comments(self.current_file.readline())

        while line and not line.startswith('*'):
            data_entry_elements = line.split(';')[:-1] # split and also strip newline character at the end

            if len(data_entry_elements) == len(header):
            # this is kind of a big stretch... I assume that x is always first
                for i in range(len(header)):
                    single_element = data_entry_elements[i]

                    if header[i] == 'x': # independent variables
                        r = re.search('(?P<low>[0-9]+\.?[0-9]*)( +TO +(?P<height>[0-9]+\.?[0-9]*))?', single_element)
                        if r:
                            if r.group('height') is not None:
                                single_element = {'low': r.group('low'), 'height': r.group('height')}
                            else:
                                single_element = r.group('low')
                            self.current_table.data['independent_variables'][xy_mapping[i]]['values'].append(single_element)

                    elif header[i] == 'y': # independant variable
                        r = re.search('( *)(?P<value>[0-9]+\.?[0-9]*)( *)\+(?P<err_p>[0-9]+\.?[0-9]*),(?P<err_m>\-[0-9]+\.?[0-9]*)( *)\((?P<err_sys>[^()]*)\)', single_element)
                        element = {'errors': []}
                        if r:
                            element['value'] = r.group('value')
                            element['errors'] += [{'label': 'stat', 'asymerror': {'plus': r.group('err_p'), 'minus': r.group('err_m')}}]

                        else:
                            r = re.search('( *)(?P<value>[0-9]+\.?[0-9]*)( *)\+-( *)(?P<error>[0-9]+\.?[0-9]*)( *)\((?P<err_sys>[^()]*)\)', single_element)
                            if r:
                                element['value'] = r.group('value')
                                element['errors'] += [{'label': 'stat', 'symerror': r.group('error')}]

                        err_sys = r.group('err_sys').split('DSYS=')

                        for err in err_sys:
                            if not err:
                                continue
                            error = {}
                            label = 'sys'
                            r = re.search('^(?P<error>[0-9]+\.?[0-9]*)(\:(?P<label>.*))?,?$', err)
                            if r:
                                if r.group('label'):
                                    label += ','+r.group('label')
                                error = {'symerror': r.group('error') }

                            else:
                                #TODO - check some assumptions - will '+' always be first? and '-' always second?
                                r = re.search('\+(?P<err_p>[0-9]+\.?[0-9]*)\,(?P<err_m>\-[0-9]+\.?[0-9]*)(\:(?P<label>.*))?', err)
                                if r:
                                    if r.group('label'):
                                        label += ','+r.group('label')
                                    error = {'asymerror': {'plus': r.group('err_p'), 'minus': r.group('err_m')}}
                            if not r:
                                # error happened
                                raise ValueError("Error while parsing data")

                            error['label'] = label
                            element['errors'].append(error)
                        self.current_table.data['dependent_variables'][xy_mapping[i]]['values'].append(element)

            last_index = self.current_file.pos
            l = self.current_file.readline()
            line = self._strip_comments(l)

        self.current_file.seek(last_index)

    def _parse_reackey(self, data):
        """Parse reackey attribute of the old HEPData format

        example:
        *reackey: P P --> Z0 Z0 X

        :param data: data to be parsed
        :type data: str
        """
        self.current_table.reactions.append(data)

    def _parse_obskey(self, data):
        """Parse obskey attribute of the old HEPData format

        example:
        *obskey: DSIG/DPT

        :param data: data to be parsed
        :type data: str
        """
        self.current_table.observables.append(data)

    def _parse_energies(self, data):
        """Add energy given in data to tables energies
        this method is here for compleatness sake, it's used in only one other place so
        can be safely extracted

        :param data: data to be appended to table's energies
        :type data: str
        """
        self.current_table.energies.append(data)

    def _parse_qual(self, data):
        """Parse qual attribute of the old HEPData format

        example qual:
        *qual: RE : P P --> Z0 Z0 X

        :param data: data to be parsed
        :type data: str
        """
        list = []
        headers = data.split(':')
        name = headers[0].strip()

        name = name.split(' IN ')
        units = None
        if len(name) > 1:
            units = name[1].strip()
        name = name[0].strip()

        for header in headers[1:]:
            xheader = {'name': name}
            if units:
                xheader['units'] = units

            xheader['value'] = header.strip()
            list.append(xheader)

            #:TODO: VERY RISKY, check energies and so on
            #this is a hack, I hope there is better way to do this:
            if name == 'SQRT(S)' and lower(units) in ('gev', 'tev', 'mev'):
                if xheader['value'] not in self.current_table.energies:
                    self._parse_energies(xheader['value'])

        self.current_table.qualifiers.append(list)

    def _parse_header(self, data):
        """Parse header (xheader or yheader)

        :param data: data to be parsed
        :type data: str
        :return: list with header's data
        :rtype: list
        """
        return_list = []

        headers = data.split(':')

        for header in headers:
            header = header.split(' IN ')
            xheader = {'name': header[0].strip()}
            if len(header) > 1:
                xheader['units'] = header[1].strip()
            return_list.append(xheader)

        return return_list

    def _parse_xheaders(self, data):
        """parse xheaders from old HEPData format

        :param data: data with xheaders to be parsed
        :type data: str
        """
        self.current_table.xheaders += self._parse_header(data)

    def _parse_yheaders(self, data):
        """parse yheaders from old HEPData format

        :param data: data with yheaders to be parsed
        :type data: str
        """
        self.current_table.yheaders += self._parse_header(data)

    @staticmethod
    def _strip_comments(line):
        """Processes line stripping any comments from it

        :param line: line to be processed
        :type line: str
        :return: line with removed comments
        :rtype: str
        """
        if line == '':
            return line
        r = re.search('(?P<line>[^#]*)(#(?P<comment>.*))?', line)
        if r:
            line = r.group('line')
            if not line.endswith('\n'):
                line += '\n'
            return line
        return '\n'

    def _read_multiline(self, init_data):
        """Reads multiline symbols (ususally comments)

        :param init_data: initial data (parsed from the line containing keyword)
        :return: parsed value of the multiline symbol
        :rtype: str
        """
        result = init_data

        while True:
            last_index = self.current_file.pos
            line_raw = self.current_file.readline()

            # don't add newlines from full line comments
            if line_raw[0] == '#':
                continue

            # now strip comments
            # TODO - is it aproperiate behavior?
            data = self._strip_comments(line_raw)
            # EOF, stop here
            if not data:
                break

            # we arrived to the next command, step back and break
            if len(data.strip()) >= 1 and data.strip()[0] == '*':
                self.current_file.seek(last_index)
                break

            result += data
        return result

    def _parse_document_comment(self, data):
        self.data[0]['comment'] = self._read_multiline(data)

    def _bind_set_table_metadata(self, key,  multiline=False):
        """Returns parsing function which will parse data as text, and add it to the table metatadata dictionary
        with the provided key

        :param key: dictionary key under which parsed data will be added to table.metadata
        :type key: str
        :param multiline: if True this attribute will be treated as multiline
        :type multiline: bool
        :return: function with bound key and multiline attributes
        :rtype: Function
        """
        def set_table_metadata(self, data):
            if multiline:
                data = self._read_multiline(data)
            self.current_table.metadata[key] = data

        # method must be bound, so we use __get__
        return set_table_metadata.__get__(self)

    def _bind_parse_record_ids(self, key):
        def _parse_record_ids(self, data):
            if 'record_ids' not in self.data[0]:
                self.data[0]['record_ids'] = []

            record_id = {key, data}

            if self.data[0]['record_ids'].count(record_id) == 0:
                self.data[0]['record_ids'].append(record_id)
            elif self.strict:
                raise BadFormat("duplicated record: *%s" % key)

        # method must be bound, so we use __get__
        return _parse_record_ids.__get__(self)

    def _bind_parse_addtional_data(self, key, multiline=False):
        """Returns parsing function which will parse data as text, and add it to the table additional data dictionary
        with the provided key

        :param key: dictionary key under which parsed data will be added to table.metadata
        :type key: str
        :param multiline: if True this attribute will be treated as multiline
        :type multiline: bool
        :return: function with bound key and multiline attributes
        :rtype: Function
        """

        def _set_additional_data_bound(self, data):
            """Concrete method for setting additional data
            :param self:
            :type self: OldHEPData
            """
            # if it's multiline, parse it
            if multiline:
                data = self._read_multiline(data)

            if key not in self.additional_data:
                self.additional_data[key] = []
            self.additional_data[key].append(data)

        # method must be bound, so we use __get__
        return _set_additional_data_bound.__get__(self)

    # employing full fledged FSM for only two states is kind of an overkill, so
    # I'll just define them here...
    states = ['document', 'table']



import copy
import csv
from hepdata_converter.writers.array_writer import ArrayWriter


class YODA(ArrayWriter):
    def __init__(self, *args, **kwargs):
        super(YODA, self).__init__(single_file_output=True, *args, **kwargs)

    def _write_table(self, data_out, table):
        headers_original = []
        qualifiers_marks_original = []



        for independent_variable in table.independent_variables:
            data_original = []

            self._extract_independent_variables(table, headers_original, data_original, qualifiers_marks_original)

            for dependent_variable in table.dependent_variables:
                where = "BELLE"
                experiment = "2013"
                inspire_id = "I1245023"

                id = "%s_%s_%s" % (where, experiment, inspire_id)

                data_out.write("# BEGIN YODA_SCATTER2D /REF/%s/d%2d-x%2d-y%2d" % id)
                data_out.write("Path=/REF/%s/d01-x01-y01" % id)
                data_out.write("Type=Scatter2D")
                data_out.write("# xval   xerr-   xerr+   yval   yerr-   yerr+")

                headers = list(headers_original)
                qualifiers = {}
                qualifiers_marks = copy(qualifiers_marks_original)
                # make a copy of the original list
                data = list(data_original)

                self._parse_dependent_variable(dependent_variable, headers, qualifiers, qualifiers_marks, data)

                # arguments = [independent_variable]

                # data_out.write("\t".join(arguments))



import yaml
from hepdata_converter.common import OptionInitMixin
from hepdata_converter.parsers import Parser, ParsedData, Table
import os


class YAML(Parser):
    options = {

    }

    def __init__(self, *args, **kwargs):
        super(YAML, self).__init__(*args, **kwargs)

    def parse(self, data_in, *args, **kwargs):
        """

        :param data_in: path to submission.yaml
        :param args:
        :param kwargs:
        :raise ValueError:
        """
        if not os.path.exists(data_in):
            raise ValueError("File does not exist: %s" % data_in)

        if os.path.isdir(data_in):
            submission_filepath = os.path.join(data_in, 'submission.yaml')
            if not os.path.exists(submission_filepath):
                submission_filepath = os.path.join(data_in, 'submission.yml')
                if not os.path.exists(submission_filepath):
                    raise ValueError("No submission file in %s" % data_in)
            data_in = submission_filepath

        with open(data_in, 'r') as submission_file:
            submission_data = list(yaml.load_all(submission_file))

        if len(submission_data) == 0:
            raise RuntimeError("Submission file (%s) is empty" % data_in)

        tables = []

        for i in range(1, len(submission_data)):
            table_filepath = os.path.join(os.path.dirname(data_in), submission_data[i]['data_file'])
            if not os.path.exists(table_filepath):
                raise ValueError("table file: %s does not exist" % table.data_file)

            with open(table_filepath, 'r') as table_file:
                table_data = yaml.load(table_file)

            table = Table(index=i, metadata=submission_data[i], data=table_data)
            tables.append(table)

        return ParsedData(submission_data[0], tables)
